@@ -80,6 +80,9 @@ extern void NSR_RemoveAllSources(void);
 /* Procedure to try to find a replacement for a bad source */
 extern void NSR_HandleBadSource(IPAddr *address);
 
+/* Procedure to resolve all names again */
+extern void NSR_RefreshAddresses(void);
+
 /* This routine is called by ntp_io when a new packet arrives off the network */
 extern void NSR_ProcessReceive(NTP_Packet *message, struct timeval *now, double now_err, NTP_Remote_Address *remote_addr, NTP_Local_Address *local_addr, int length);
 
